@@ -13,7 +13,7 @@ import time
 import cx_Oracle
 import MySQLdb.cursors
 
-from .constants import (APP_CHARSET, IS_PYTHON_3, TMP_FOLDER,
+from .constants import (IS_PYTHON_3, TMP_FOLDER,
                         MARIA_CHARSET, MARIA_COLLATE, MARIA_STORAGE)
 from .aes import AESCipher
 from .services import DB_INSTANCES, DB_CREDENTIALS
@@ -72,7 +72,7 @@ class Database(object):
                 passwd=password,
                 db=self.database if existent else 'information_schema',
                 cursorclass=MySQLdb.cursors.SSCursor,
-                charset=APP_CHARSET,
+                charset=MARIA_CHARSET,
                 use_unicode=IS_PYTHON_3,
                 local_infile=1 * self.local_infile
             )
@@ -230,8 +230,8 @@ class Database(object):
             values = ', '.join(values)
             sql = 'insert into {} VALUES ({})'.format(table, values)
             if chunk:
-                for data in [iterable[i:i + chunk]
-                             for i in range(0, len(iterable), chunk)]:
+                for data in (iterable[i:i + chunk]
+                             for i in range(0, len(iterable), chunk)):
                     self._list_to_table_oracle(sql, data)
             else:
                 self._list_to_table_oracle(sql, iterable)
@@ -256,7 +256,7 @@ class Database(object):
             filename=filename,
             table=table,
             partition='PARTITION ({})'.format(partition) if partition else '',
-            charset=APP_CHARSET,
+            charset=MARIA_CHARSET,
             delimiter=delimiter,
             endline=endline
         )
@@ -444,19 +444,20 @@ class Database(object):
                 ('ora', 'DATE', 'ora', 'query'): column,
                 ('ora', 'DATE_J', 'my', 'create'): '{column} date',
                 ('ora', 'DATE_J', 'my', 'query'):
-                    "to_char(to_date({column}, 'J'), 'YYYYMMDD')",
+                    "case when {column} > 3000000 then null else to_char(to_date({column}, 'J'), 'YYYYMMDD') end",  # noqa
                 ('ora', 'DATE_J', 'ora', 'create'): '{column} date',
-                ('ora', 'DATE_J', 'ora', 'query'): "to_date({column}, 'J')",
+                ('ora', 'DATE_J', 'ora', 'query'):
+                    "case when {column} > 3000000 then null else to_date({column}, 'J') end",  # noqa
                 ('ora', 'DATETIME', 'my', 'create'): '{column} datetime',
                 ('ora', 'DATETIME', 'my', 'query'):
                     "to_char({column}, 'YYYY-MM-DD HH24:MI:SS')",
                 ('ora', 'DATETIME', 'ora', 'create'): '{column} date',
                 ('ora', 'DATETIME', 'ora', 'query'): column,
-                ('ora', 'TIMESTAMP', 'my', 'create'): '{column} datetime',
-                ('ora', 'TIMESTAMP', 'my', 'query'):
+                ('ora', 'TIMESTAMP(6)', 'my', 'create'): '{column} datetime',
+                ('ora', 'TIMESTAMP(6)', 'my', 'query'):
                     "to_char({column}, 'YYYY-MM-DD HH24:MI:SS')",
-                ('ora', 'TIMESTAMP', 'ora', 'create'): '{column} date',
-                ('ora', 'TIMESTAMP', 'ora', 'query'): column,
+                ('ora', 'TIMESTAMP(6)', 'ora', 'create'): '{column} date',
+                ('ora', 'TIMESTAMP(6)', 'ora', 'query'): column,
                 ('ora', 'NUMBER', 'my', 'create'):
                     '{column} double' if scale
                     else '{column} int' if not precision or precision < 10
@@ -469,17 +470,26 @@ class Database(object):
                 ('ora', 'VARCHAR2', 'my', 'create'):
                     "{column} varchar({length})",
                 ('ora', 'VARCHAR2', 'my', 'query'):
-                    'ltrim(rtrim({column}))' if length > 49 else column,
+                    'ltrim(rtrim({column}))' if length and length > 49 else column,  # noqa
                 ('ora', 'VARCHAR2', 'ora', 'create'):
                     '{column} varchar2({length})',
                 ('ora', 'VARCHAR2', 'ora', 'query'): column,
                 ('ora', 'CHAR', 'my', 'create'):
                     "{column} varchar({length})",
                 ('ora', 'CHAR', 'my', 'query'):
-                    'ltrim(rtrim({column}))' if length > 49 else column,
+                    'ltrim(rtrim({column}))' if length and length > 49 else column,  # noqa
                 ('ora', 'CHAR', 'ora', 'create'):
                     '{column} char({length})',
-                ('ora', 'CHAR', 'ora', 'query'): column}
+                ('ora', 'CHAR', 'ora', 'query'): column,
+                ('ora', 'NVARCHAR2', 'my', 'create'):
+                    "{column} varchar({length})",
+                ('ora', 'NVARCHAR2', 'my', 'query'):
+                    'ltrim(rtrim({column}))' if length and length > 49 else column,  # noqa
+                ('ora', 'NVARCHAR2', 'ora', 'create'):
+                    '{column} nvarchar2({length})',
+                ('ora', 'NVARCHAR2', 'ora', 'query'): column,
+                ('ora', 'NCLOB', 'ora', 'create'): '{column} nclob',
+                ('ora', 'NCLOB', 'ora', 'query'): column}
         resul = {key: conv[(self.engine, type, desti, key)].format(**param)
                  for key in ('create', 'query')}
         return resul
